#!/bin/bash

#/*******************************************************************
# * This file is part of the Emulex Linux Device Driver for         *
# * Fibre Channel Host Bus Adapters.                                *
# * Copyright (C) 2012 Emulex.  All rights reserved.                *
# * EMULEX and SLI are trademarks of Emulex.                        *
# * www.emulex.com                                                  *
# *                                                                 *
# * This program is free software; you can redistribute it and/or   *
# * modify it under the terms of version 2 of the GNU General       *
# * Public License as published by the Free Software Foundation.    *
# * This program is distributed in the hope that it will be useful. *
# * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
# * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
# * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
# * DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
# * TO BE LEGALLY INVALID.  See the GNU General Public License for  *
# * more details, a copy of which can be found in the file COPYING  *
# * included with this package.                                     *
# *******************************************************************/

######################################################################
#                          elx_iscsi_install.sh                      #
# This installation script installs, uninstalls or loads the         #
# appropriate Emulex Linux iscsi device driver as required for       #
# supporting the Emulex Universal CNAs. In addition it updates the   #
# SLES /etc/sysconfig/kernel as needed.                              #
#                                                                    #
######################################################################

SYSCONFIG_KERNEL_BACKUP=/etc/sysconfig/kernel.elx
SYSCONFIG_KERNEL_TEMP=/etc/sysconfig/kernel.be2iscsi
DRIVER_NAME=be2iscsi
EXTRA_OPTS=""
RPM_PKG_INSTALLED=""
KMOD_KMP=""

####################################################################
#                                                                  #
# Function: usage                                                  #
#                                                                  #
# Display information about the usage of this script.              #
#                                                                  #
####################################################################
function usage()
{
    echo ""
    echo "$0 if invoked without parameters will by default install"
    echo "The following options may also be used:"
    echo ""
    echo "   -i, --install      - Installs driver."
    echo "   -u, --uninstall    - Uninstall previously installed driver."
    echo "   -h, --help         - Prints help message describing command line parameters."
    echo ""
}

#####################################################################
#                                                                   #
# Function: config_sles_sysconfig_kernel                            #
#                                                                   #
# Configure the /etc/sysconfig/kernel file on SLES installation to  #
# allow the driver to be part of the initial ramdisk when rebuilt,  #
# and load it on system boot.                                       #
#                                                                   #
# Returns:                                                          #
#         0 -- /etc/sysconfig/kernel file not modified              #
#         1 -- /etc/sysconfig/kernel file modified                  #
#                                                                   #
#####################################################################
function config_sles_sysconfig_kernel()
{
    local RET_VALUE
    RET_VALUE=0

    if [ ${DIST:0:5} = "SLES1" ] ; then
	echo "Configuring /etc/sysconfig/kernel ..."
	if [ $(grep -c -e "^[[:space:]]*INITRD_MODULES[[:space:]]*=[[:space:]]*.*[\"[:space:]]${DRIVER_NAME}[\"[:space:]]" /etc/sysconfig/kernel) -eq 0 ] ; then
	    cp /etc/sysconfig/kernel ${SYSCONFIG_KERNEL_BACKUP}
	    echo "Original saved as ${SYSCONFIG_KERNEL_BACKUP}"
	    sed "s/^INITRD_MODULES=\"\(.*\)\"/INITRD_MODULES=\"\1 ${DRIVER_NAME}\"/" /etc/sysconfig/kernel > ${SYSCONFIG_KERNEL_TEMP}
	    mv -f ${SYSCONFIG_KERNEL_TEMP} /etc/sysconfig/kernel
	    let RET_VALUE=1
	fi
    fi

    return ${RET_VAL}
}

#####################################################################
#                                                                   #
# Function: unconfig_sles_sysconfig_kernel                          #
#                                                                   #
# Configure the /etc/sysconfig/kernel file on SLES installation to  #
# remove the driver from the initial ramdisk when rebuilt, and not  #
# load it on system boot.                                           #
#                                                                   #
# Returns:                                                          #
#         0 -- /etc/sysconfig/kernel file not modified              #
#         1 -- /etc/sysconfig/kernel file modified                  #
#                                                                   #
#####################################################################
function unconfig_sles_sysconfig_kernel()
{
    local MOD_FILE
    local RET_VALUE
    MOD_FILE=0
    RET_VALUE=0

    if [ ${DIST:0:5} = "SLES1" ] ; then
	echo "Configuring /etc/sysconfig/kernel ..."
	cp -f /etc/sysconfig/kernel ${SYSCONFIG_KERNEL_BACKUP}

	if [ $(grep -c -e "^[[:space:]]*INITRD_MODULES[[:space:]]*=[[:space:]]*.*[\"[:space:]]${DRIVER_NAME}[\"[:space:]]" /etc/sysconfig/kernel) -ne 0 ] ; then
	    sed "/^[[:space:]]*INITRD_MODULES[[:space:]]*=.*\".*/ {s/\([\"[:space:]]\)${DRIVER_NAME}\([\"[:space:]]\)/\1\2/g}" /etc/sysconfig/kernel > ${SYSCONFIG_KERNEL_TEMP}

	    mv -f ${SYSCONFIG_KERNEL_TEMP} /etc/sysconfig/kernel
	    MOD_FILE=$((${MOD_FILE}+1))
	    RET_VALUE=1
	fi

	if [ ${MOD_FILE} -ne 0 ] ; then
	    echo "Original saved as ${SYSCONFIG_KERNEL_BACKUP}"
	else
	    rm ${SYSCONFIG_KERNEL_BACKUP}
	fi
    fi

    return ${RET_VALUE}
}

#####################################################################
#                                                                   #
# Function: update_depmod                                           #
# Update depmod.conf files to load be2iscsi driver first            #
#                                                                   #
# Returns:                                                          #
#         0 -- Success                                              #
#         1 -- Failure                                              #
#                                                                   #
#####################################################################
function update_depmod
{
    if [ "$DIST" == "SLES10" ] ; then
        # Nothing is necessary for SLES10
        return 0
    fi

    if [ "`cat /etc/depmod.d/elxbe2iscsi.conf 2>/dev/null | grep "override be2iscsi"`" == "" ]; then
        echo "override be2iscsi * weak-updates/updates/be2iscsi" >> /etc/depmod.d/elxbe2iscsi.conf
        echo "override be2iscsi * weak-updates/be2iscsi" >> /etc/depmod.d/elxbe2iscsi.conf
    fi
    return 0
}

#####################################################################
#                                                                   #
# Function: clean_depmod                                            #
# Cleanup depmod.conf files to remove be2iscsi references           #
#                                                                   #
# Returns:                                                          #
#         0 -- Success                                              #
#         1 -- Failure                                              #
#                                                                   #
#####################################################################
function clean_depmod
{
    if [ "$DIST" == "SLES10" ] ; then
        # Nothing is necessary for SLES10
        return 0
    fi

    rm -f /etc/depmod.d/elxbe2iscsi.conf

    return 0
}

#####################################################################
#                                                                   #
# Function: install                                                 #
#                                                                   #
# Unpack the iscsi driver package and install the driver binary RPM.#
#                                                                   #
# Returns:                                                          #
#         0 -- Success                                              #
#         1 -- Error in driver RPM installation                     #
#                                                                   #
#####################################################################
function install()
{
    echo "Unpacking be2iscsi installation files..."
    tar xf be2iscsi*.tar.gz

    DIR=${DIST}${ARCH_STR}

    # Find and install the proper RPM 

    # set nullglob so the we get no expansion if the glob fails.
    # Assign to an array so the expansion can be done inline.
    shopt -s nullglob
    # After the expansion of ${KERN_VAR}- we can have up to three choices
    # It can be either a 2, a 4 or a 10.
    PKG=( ${DIR}/${RPM_BASE_NAME}${KERN_VAR}-{2,4,10}.*_${RPM_DISTRO}-[1-9].${RPM_ARCH_STR}.rpm )
    shopt -u nullglob
    # Check for whether the expansion failed.
    if [[ ${#PKG[@]} -eq 0 ]]
    then
        echo "ERROR:rpm file not found."
        exit 1
    fi
    # Check if we got too many matches.
    if [[ ${#PKG[@]} -gt 1 ]]
    then
        echo "INTERNAL ERROR: Too many rpm files found."
        echo "Matching packages: ${PKG[@]}"
        exit 1
    fi
    # Use the one true package.
    pkg="${PKG[0]}"
    echo "Installing be2iscsi package, ${pkg} ..."
    
    #Fix whitelist issue with SLES11 SP1 as well as RHEL
    if [[ `echo ${RPM_DISTRO} | grep "2.6.32"` != "" || `echo ${DIST} | grep RHEL` != "" ]]; then
        rpm -Uhv --nodeps ${EXTRA_OPTS} ${pkg}
    else
        rpm -Uhv ${EXTRA_OPTS} ${pkg}
    fi

    if [ $? -ne 0 ] ; then
        echo "ERROR: RPM installation failed"
        exit 1
    fi
}

#####################################################################
#                                                                   #
# Function: load_driver                                             #
#                                                                   #
# Load the iscsi driver using the modprobe command.                 #
#                                                                   #
# Returns:                                                          #
#         0 -- Driver was loaded successfully                       #
#         1 -- Error in loading driver                              #
#                                                                   #
#####################################################################
function load_driver()
{
    modinfo ${DRIVER_NAME} > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        modprobe ${DRIVER_NAME}
        if [ $? -eq 0 ]; then
            echo "Loaded driver ${DRIVER_NAME}"
            return 0
        else
            echo "Failed to load driver ${DRIVER_NAME}"
            return 1
        fi
    fi

    return 0
}

#####################################################################
#                                                                   #
# Function: unload_driver                                           #
#                                                                   #
# Unload the iscsi driver using the "modprobe -r" command.          #
#                                                                   #
# Returns:                                                          #
#         0 -- Driver was unloaded successfully                     #
#         1 -- Error in unloading driver                            #
#                                                                   #
#####################################################################
function unload_driver()
{
    RET_VAL=0

    lsmod | grep "${DRIVER_NAME}" > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        modprobe -r ${DRIVER_NAME}
        if [ $? -eq 0 ]; then
            echo "Unloaded driver ${DRIVER_NAME}"
            RET_VAL=0
        else
	    echo "Warning: ${DRIVER_NAME} is being used and cannot be unloaded"
	    RET_VAL=1
        fi
    fi

    return ${RET_VAL}
}

#
# Main script process starts here
#

# Find kernel variant
CARCH=$(uname -m)       # Canonical machine arch.
UNAME=$(uname -r)
if [[ -f /etc/asianux-release ]]
then
    if [[ "$CARCH" == ia64 ]]
    then
        KERNEL_IMAGE_PATH="/boot/efi/efi/asianux/vmlinuz-$UNAME"
    else
        KERNEL_IMAGE_PATH="/boot/vmlinuz-$UNAME"
    fi
elif [[ -f /etc/redhat-release || -f /etc/centos-release ]]
then
    if [[ "$CARCH" == ia64 ]]
    then
	KERNEL_IMAGE_PATH="/boot/efi/efi/redhat/vmlinuz-$UNAME"
    else
	KERNEL_IMAGE_PATH="/boot/vmlinuz-$UNAME"
    fi
elif [[ -f /etc/SuSE-release ]]
then
    if [[ "$CARCH" == ppc64 ]]
    then
	KERNEL_IMAGE_PATH="/boot/vmlinux"
    else
	KERNEL_IMAGE_PATH="/boot/vmlinuz-$UNAME"
    fi
else
    echo ""
    echo "Unable to determine distribution type"
    exit 1
fi

KERN_VAR_LIST=($(rpm -qf $KERNEL_IMAGE_PATH --qf "%{NAME}\n"))
if (( ${#KERN_VAR_LIST[@]} > 1 ))
then
    # Check if the KERNEL_IMAGE_PATH is a symlink
    if [[ -h "$KERNEL_IMAGE_PATH" ]]
    then
        # If it is then resolve what it points to.
        # Use -e to ensure that the link points to something that exists.
        if ! REAL_KERNEL_IMAGE_PATH=$(readlink -e $KERNEL_IMAGE_PATH)
        then
            echo "ERROR: $KERNEL_IMAGE_PATH does not exist"
            exit 1
        fi
        # Check if it corresponds to the running kernel
        if [[ ${REAL_KERNEL_IMAGE_PATH%$UNAME} != ${REAL_KERNEL_IMAGE_PATH} ]]
        then
            KERNEL_IMAGE_PATH=$REAL_KERNEL_IMAGE_PATH
            KERN_VAR_LIST=($(rpm -qf $KERNEL_IMAGE_PATH --qf "%{NAME}\n"))
            if (( ${#KERN_VAR_LIST[@]} > 1 ))
            then
                echo "ERROR: Multiple packages own $KERNEL_IMAGE_PATH"
                exit 1
            fi
        else
            echo "ERROR: the installed kernel does not mach the running kernel"
            exit 1
        fi
    else
        echo "ERROR: the installed kernel could not be determined"
        exit 1
    fi
fi
KERN_VAR=$(echo "${KERN_VAR_LIST[0]}" | \
    sed 's/kernel\(.*\)/\1/' | sed 's/-uek\(.*\)/\1/' | sed 's/-base\(.*\)/\1/')

# Determine distribution version
# IMPORTANT NOTE:
# Setting the RPM_DISTRO variable for SLES requires a star at the
# end surrounded by quotes. (Single quotes are good, double quotes are
# less stylish).
# The star will change how the rpm is selected for sles *because* we
# need extra globbing. The globing is specifically turned *off* for RHEL.
if [ $(rpm -q sles-release 2>/dev/null 1>/dev/null ; echo $?) -eq 0 ] ; then
    # SLES
    # Note that kernel_pt_rev means something different in the Rev3
    # kernel than in Rev2.6
    # Kernel Rev 3 example:
    # Given X.Y.Z-A.B.C
    #       | | | | | + KERN_PT_MICRO (unused)
    #       | | | | + KERNL_PT_MINOR (we call it kernel_pt_rev)
    #       | | | + KERN_PT_MAJOR
    #       | | + KERN_REVISION == MAJ_DISTRO
    #       | + KERN_MINOR
    #       + KERN_MAJOR
    KERN_MAJOR=$(echo $UNAME | awk -F- '{print $1}' | awk -F. '{print $1}')
    KERN_MINOR=$(echo $UNAME | awk -F- '{print $1}' | awk -F. '{print $2}')
    KERN_REVISION=$(echo $UNAME | awk -F- '{print$1}' | awk -F. '{print$3}')
    MAJ_DISTRO=$KERN_REVISION
    if [ $KERN_MAJOR -eq 3 ] ; then
        if [ $KERN_MINOR -eq 0 ] ; then
            # 3.0.X kernel
            DIST=SLES11
            if [ $KERN_REVISION -ge 13 ]; then
                RPM_DISTRO=3.0.
            else
                echo "Error: running kernel not supported." ; exit 1
            fi
        else
            echo "Error: running kernel not supported." ; exit 1
        fi
    elif [ $MAJ_DISTRO -eq 32 ] ; then
        # Kernel 2.6.32
        DIST=SLES11
        kernel_pt_rev="$(uname -r | awk -F- '{print$1}' | awk -F. '{print$4}')"
        if [ $kernel_pt_rev -ge 12 ] && [ $kernel_pt_rev -lt 36 ]; then
            RPM_DISTRO=2.6.32.12
        elif [  $kernel_pt_rev -ge 36 ]; then
            RPM_DISTRO=2.6.32.36
        else
            echo "Error: running kernel not supported." ; exit 1
        fi
    elif [ $MAJ_DISTRO -eq 27 ] ; then
        # Kernel 2.6.27
        DIST=SLES11
	RPM_DISTRO=2.6.27.19
    elif [ $MAJ_DISTRO -eq 16 ] ; then
        # Kernel 2.6.16
        DIST=SLES10
        for kernel_pt_rev in "$(uname -r | awk -F- '{print$1}' | awk -F. '{print$4}')" ; do
	    case $kernel_pt_rev in
		60) SLES_SUBREV=$(uname -r | awk -F- '{print$2}' | awk -F. '{print$2}')
		    if [ $SLES_SUBREV -ge 54 ] && [ $SLES_SUBREV -lt 85 ]; then
			RPM_DISTRO=2.6.16.60_0.54
		    elif [ $SLES_SUBREV -ge 85 ] ; then
			RPM_DISTRO=2.6.16.60_0.85
		    fi
		    ;;
		*) echo "Error: running kernel not supported." ; exit 1 ;;
	    esac
	done
    fi

    KMOD_KMP=kmp
    RPM_BASE_NAME=be2iscsi-${KMOD_KMP}
    # IMPORTANT NOTE:
    # Setting the RPM_DISTRO variable for SLES requires a star at the
    # end.
    # The star will change how the rpm is selected for sles *because* we
    # need extra globbing. The globbing is specifically turned *off* for RHEL.
    RPM_DISTRO="${RPM_DISTRO}*"

    # RPM package name; will be used when deleting the RPM during the uninstall
    # phase.
    RPM_PKG_INSTALLED=be2iscsi-kmp${KERN_VAR}

elif rpm -q oraclelinux-release > /dev/null 2>&1
then
    # oel-5.6-UEK:    2.6.32-100.26.2.el5uek
    # oel-5.7-UEK:    2.6.32-200.13.1.el5uek
    # oel-5.8-UEK:    2.6.32-300.10.1.el5uek

    # oel-6.0-UEK:    2.6.32-100.28.5.el6
    # oel-6.1-UEK:    2.6.32-100.34.1.el6uek
    # oel-6.2-UEK:    2.6.32-300.3.1.el6uek
    # oel-6.3-UEK:    2.6.39-200.24.1.el6uek
    KMOD_KMP=kmod-
    MAJ_DISTRO=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$3}')

    if [[ $MAJ_DISTRO -eq 32 || $MAJ_DISTRO -eq 39 ]]
    then
        kernel_pt_rev=$(echo $UNAME | awk -F- '{split($2, a, "."); print a[1]}')
        kernel_pt_rev2=$(echo $UNAME | awk -F- '{split($2, a, "."); print a[2]}')
    fi
    if [ $MAJ_DISTRO -eq 32 ] ; then

        if [[ $kernel_pt_rev -eq 300 ]]; then
            if [[ $kernel_pt_rev2 -eq 3 ]]; then
                DIST=OL6_UEK
	        RPM_DISTRO=uek6u2
            elif [[ $kernel_pt_rev2 -eq 10 ]]; then
	        DIST=OL5_UEK
	        RPM_DISTRO=uek5u8
                # Comment this out for now, since the raw driver bRPMs still
                # have the "kmod-" in their name. That would/should eventually
                # be removed for OL5-UEK bRPMs, at which point the following
                # line should be un-commented.
                KMOD_KMP=
            fi
        elif [[ $kernel_pt_rev -eq 200 ]]; then
	    DIST=OL5_UEK
	    RPM_DISTRO=uek5u7
            # Comment this out for now, since the raw driver bRPMs still
            # have the "kmod-" in their name. That would/should eventually
            # be removed for OL5-UEK bRPMs, at which point the following
            # line should be un-commented.
            KMOD_KMP=
        elif [[ $kernel_pt_rev -eq 100 ]]; then
            if [[ $kernel_pt_rev2 -eq 34 ]]; then
                DIST=OL6_UEK
	        RPM_DISTRO=uek6u1
            elif [[ $kernel_pt_rev2 -eq 26 ]]; then
	        DIST=OL5_UEK
	        RPM_DISTRO=uek5u6
                # Comment this out for now, since the raw driver bRPMs still
                # have the "kmod-" in their name. That would/should eventually
                # be removed for OL5-UEK bRPMs, at which point the following
                # line should be un-commented.
                KMOD_KMP=
            elif [[ $kernel_pt_rev2 -eq 28 ]]; then
	        DIST=OL6_UEK
	        RPM_DISTRO=uek6
            fi
        fi
    elif [ $MAJ_DISTRO -eq 39 ] ; then
        if [[ $kernel_pt_rev -eq 200 ]]; then
            if [[ $kernel_pt_rev2 -eq 24 ]]; then
                DIST=OL6_UEK
                RPM_DISTRO=uek6u3
            fi
        fi
    fi

    RPM_BASE_NAME=${KMOD_KMP}be2iscsi
    # If we found something then set RPM_PKG_INSTALLED. Otherwise, we will
    # drop into the RHEL section below, because we might be running
    # a RHEL6 kernel on a UEK installation.
    if [ -n "$RPM_DISTRO" ]; then
        RPM_PKG_INSTALLED=${RPM_BASE_NAME}${KERN_VAR}
    fi
fi
if [[ -z "$RPM_PKG_INSTALLED" ]] && {
                rpm -q oraclelinux-release ||
                rpm -q enterprise-release  ||
                rpm -q redhat-release ||
                rpm -q redhat-release-server ||
                rpm -q centos-release
            } > /dev/null 2>&1
then
    # RHEL
    MAJ_DISTRO=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$3}')
    if [ $MAJ_DISTRO -eq 32 ] ; then
	# Kernel 2.6.32
	DIST=RHEL6
        kernel_pt_rev=$(uname -r | awk -F- '{split($2, a, "."); print a[1]}')
        if [[ $kernel_pt_rev -ge 71 && $kernel_pt_rev -lt 131 ]]
        then
            # RHEL-6.0
	    RPM_DISTRO=rh6
        elif [[ $kernel_pt_rev -ge 131 && $kernel_pt_rev -lt 220 ]]
        then
            # RHEL-6.1
	    RPM_DISTRO=rh6u1
        elif [[ $kernel_pt_rev -ge 220 && $kernel_pt_rev -lt 279 ]]
        then
            # RHEL-6.2
            RPM_DISTRO=rh6u2
        elif [[ $kernel_pt_rev -ge 279 ]]
        then
            # RHEL-6.3
            RPM_DISTRO=rh6u3
        else
            echo 'Error: running kernel not supported.'
            exit 1
        fi
    elif [ $MAJ_DISTRO -eq 18 ] ; then
	# Kernel 2.6.18
	DIST=RHEL5
	for kernel_pt_rev in "$(uname -r | awk -F- '{print$2}' | awk -F. '{print$1}')" ; do 
	    if [[ $kernel_pt_rev -ge 164 && $kernel_pt_rev -lt 194 ]]; then
		POINT_REL=4
	    elif [[ $kernel_pt_rev -ge 194 && $kernel_pt_rev -lt 238 ]]; then
		POINT_REL=5
	    elif [[ $kernel_pt_rev -ge 238 && $kernel_pt_rev -lt 274 ]]; then
		POINT_REL=6
	    elif [[ $kernel_pt_rev -ge 274 && $kernel_pt_rev -lt 308 ]]; then
		POINT_REL=7
	    elif [[ $kernel_pt_rev -ge 308 ]]; then
		POINT_REL=8
	    else
	        echo "Error: running kernel not supported." ; exit 1
	    fi
	done
	RPM_DISTRO=rh5u${POINT_REL}
    else
	echo "Error: running kernel not supported." ; exit 1
    fi

    # Uncomment the next line if there are any whitelist issues w/ any RHEL 
    # kernels.
#    EXTRA_OPTS="--nodeps"

    KMOD_KMP=kmod
    RPM_BASE_NAME=${KMOD_KMP}-be2iscsi

    # RPM package name; will be used when deleting the RPM during the uninstall
    # phase.
    RPM_PKG_INSTALLED=kmod-be2iscsi${KERN_VAR}
fi
if [ -z "$RPM_PKG_INSTALLED" ]; then
    echo "Error: Unable to determine distribution and version"
    exit 1
fi

# Determine processor architecture
ARCH=$(uname -m)
case ${ARCH} in
    i[3456]86)
	ARCH_STR=
	RPM_ARCH_STR=i?86
	;;
    x86_64)
	ARCH_STR=_64
	RPM_ARCH_STR=x86_64
	;;
    *)
	echo "ERROR: Unsupported architecture ${ARCH}"
	exit 2
	;;
esac

# Process command options
if [ $# -eq 0 ] ; then
    INSTALL=1
else
    while [ $# -ne 0 ] ; do
        case "$1" in
            -u|--uninstall)
                shift
                if [ ! -z "$INSTALL" ] || [ $# -ne 0 ] ; then
                    echo "Error: Uninstall must be run without any other arguments"
                    exit 1
                else
                    UNINSTALL=1
                fi
                ;;
            -i|--install)
                shift
                INSTALL=1
                ;;
            *)
                usage
                exit 1
                ;;
        esac
    done
fi

# Process command options
if [ ! -z "$INSTALL" ] ; then
    update_depmod
    install
    if [ $? -eq 0 ] ; then
        unload_driver || \
echo "Please reboot this system to complete the uninstallation"
        config_sles_sysconfig_kernel
        load_driver
    fi
    echo "Installation complete"
elif [ ! -z "$UNINSTALL" ] ; then
    unload_driver || \
echo "Please reboot this system to complete the uninstallation"

    if rpm -q ${RPM_PKG_INSTALLED} > /dev/null 2>&1
    then
        rpm -e --allmatches ${RPM_PKG_INSTALLED}
    fi

# In order to to cover the boot from scsi case, where a user might uninstall 
# the be2iscsi driver and the system becomes unbootable, we will not
# remove the iscsi driver from the initial ramdisk. 
#        unconfig_sles_sysconfig_kernel
    load_driver
else
    echo "Invalid Option"
    exit 1
fi

exit 0
